@@ -1,24 +1,23 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8; coding: utf-8 -*-
- *  test-widget.c
+ * test-widget.c
+ * This file is part of GtkSourceView
  *
- *  Copyright (C) 2001
- *  Mikael Hermansson<tyan@linux.se>
+ * Copyright (C) 2001 - Mikael Hermansson <tyan@linux.se>
+ * Copyright (C) 2003 - Gustavo Giráldez <gustavo.giraldez@gmx.net>
  *
- *  Copyright (C) 2003 - Gustavo Giráldez <gustavo.giraldez@gmx.net>
+ * GtkSourceView is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * GtkSourceView is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 /* If TEST_XML_MEM is defined the test program will try to detect memory
@@ -92,6 +91,13 @@ static void       indent_toggled_cb              (GtkAction       *action,
 						  GtkAction       *current,
 						  gpointer         user_data);
 
+static void       forward_string_cb              (GtkAction       *action,
+						  gpointer         user_data);
+
+static void       backward_string_cb             (GtkAction       *action,
+						  gpointer         user_data);
+
+
 static GtkWidget *create_view_window             (GtkSourceBuffer *buffer,
 						  GtkSourceView   *from);
 
@@ -119,6 +125,10 @@ static GtkActionEntry view_action_entries[] = {
 	  "Find", G_CALLBACK (find_cb) },
 	{ "Replace", GTK_STOCK_FIND_AND_REPLACE, "Search and _Replace", "<control>R",
 	  "Search and Replace", G_CALLBACK (replace_cb) },
+	{ "ForwardString", NULL, "_Forward to string toggle", "<control>S",
+	  "Forward to the start or end of the next string", G_CALLBACK (forward_string_cb) },
+	{ "BackwardString", NULL, "_Backward to string toggle", "<control><shift>S",
+	  "Backward to the start or end of the next string", G_CALLBACK (backward_string_cb) }
 };
 
 static GtkToggleActionEntry toggle_entries[] = {
@@ -223,6 +233,9 @@ static const gchar *view_ui_description =
 "        <menuitem action=\"SmartHomeEndAfter\"/>"
 "        <menuitem action=\"SmartHomeEndAlways\"/>"
 "      </menu>"
+"      <separator/>"
+"      <menuitem action=\"ForwardString\"/>"
+"      <menuitem action=\"BackwardString\"/>"
 "    </menu>"
 "  </menubar>"
 "</ui>";
@@ -619,6 +632,61 @@ new_view_cb (GtkAction *action, gpointer user_data)
 	gtk_widget_show (window);
 }
 
+static void
+forward_string_cb (GtkAction *action,
+		   gpointer   user_data)
+{
+	GtkSourceBuffer *buffer;
+	GtkSourceView *view;
+	GtkTextIter iter;
+	GtkTextMark *insert;
+
+	g_return_if_fail (GTK_IS_SOURCE_VIEW (user_data));
+
+	view = GTK_SOURCE_VIEW (user_data);
+	buffer = GTK_SOURCE_BUFFER (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
+	insert = gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (buffer));
+
+	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (buffer),
+	                                  &iter,
+	                                  insert);
+
+	if (gtk_source_buffer_iter_forward_to_context_class_toggle (buffer,
+	                                                            &iter,
+	                                                            "string"))
+	{
+		gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (buffer), &iter);
+		gtk_text_view_scroll_mark_onscreen (GTK_TEXT_VIEW (view), insert);
+	}
+}
+
+static void
+backward_string_cb (GtkAction *action,
+		    gpointer   user_data)
+{
+	GtkSourceBuffer *buffer;
+	GtkSourceView *view;
+	GtkTextIter iter;
+	GtkTextMark *insert;
+
+	g_return_if_fail (GTK_IS_SOURCE_VIEW (user_data));
+
+	view = GTK_SOURCE_VIEW (user_data);
+	buffer = GTK_SOURCE_BUFFER (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
+	insert = gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (buffer));
+
+	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (buffer),
+	                                  &iter,
+	                                  insert);
+
+	if (gtk_source_buffer_iter_backward_to_context_class_toggle (buffer,
+	                                                             &iter,
+	                                                             "string"))
+	{
+		gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (buffer), &iter);
+		gtk_text_view_scroll_mark_onscreen (GTK_TEXT_VIEW (view), insert);
+	}
+}
 
 /* Buffer action callbacks ------------------------------------------------------------ */
 
@@ -658,14 +726,14 @@ search_dialog (GtkWidget            *widget,
 			       "text", search_data.what ? search_data.what : "",
 			       "activates-default", TRUE,
 			       NULL);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
+	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
 				     GTK_WIDGET (entry1), TRUE, TRUE, 0);
 	entry2 = g_object_new (GTK_TYPE_ENTRY,
 			       "visible", replace,
 			       "text", search_data.replacement ? search_data.replacement : "",
 			       "activates-default", TRUE,
 			       NULL);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
+	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
 				     GTK_WIDGET (entry2), TRUE, TRUE, 0);
 
 	case_sensitive = g_object_new (GTK_TYPE_CHECK_BUTTON,
@@ -673,7 +741,7 @@ search_dialog (GtkWidget            *widget,
 				       "label", "Case sensitive",
 				       "active", !(search_data.flags & GTK_SOURCE_SEARCH_CASE_INSENSITIVE),
 				       NULL);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
+	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
 			    GTK_WIDGET (case_sensitive), FALSE, FALSE, 0);
 
 	while (TRUE)
@@ -1026,15 +1094,18 @@ static void
 update_cursor_position (GtkTextBuffer *buffer, gpointer user_data)
 {
 	gchar *msg;
-	gint row, col, chars, tabwidth;
-	GtkTextIter iter, start;
+	gint row, chars;
+	guint col;
+	GtkTextIter iter;
 	GtkSourceView *view;
 	GtkLabel *pos_label;
+	gchar **classes;
+	gchar **classes_ptr;
+	GString *str;
 
 	g_return_if_fail (GTK_IS_SOURCE_VIEW (user_data));
 
 	view = GTK_SOURCE_VIEW (user_data);
-	tabwidth = gtk_source_view_get_tab_width (view);
 	pos_label = GTK_LABEL (g_object_get_data (G_OBJECT (view), "pos_label"));
 
 	gtk_text_buffer_get_iter_at_mark (buffer,
@@ -1043,26 +1114,32 @@ update_cursor_position (GtkTextBuffer *buffer, gpointer user_data)
 
 	chars = gtk_text_iter_get_offset (&iter);
 	row = gtk_text_iter_get_line (&iter) + 1;
+	col = gtk_source_view_get_visual_column (view, &iter) + 1;
 
-	start = iter;
-	gtk_text_iter_set_line_offset (&start, 0);
-	col = 0;
+	classes = gtk_source_buffer_get_context_classes_at_iter (GTK_SOURCE_BUFFER (buffer),
+	                                                         &iter);
 
-	while (!gtk_text_iter_equal (&start, &iter))
+	str = g_string_new ("");
+	classes_ptr = classes;
+
+	while (classes_ptr && *classes_ptr)
 	{
-		if (gtk_text_iter_get_char (&start) == '\t')
+		if (classes_ptr != classes)
 		{
-			col += (tabwidth - (col % tabwidth));
+			g_string_append (str, ", ");
 		}
-		else
-			++col;
 
-		gtk_text_iter_forward_char (&start);
+		g_string_append_printf (str, "%s", *classes_ptr);
+		++classes_ptr;
 	}
 
-	msg = g_strdup_printf ("char: %d, line: %d, column: %d", chars, row, col);
+	g_strfreev (classes);
+
+	msg = g_strdup_printf ("char: %d, line: %d, column: %d, classes: %s", chars, row, col, str->str);
 	gtk_label_set_text (pos_label, msg);
-      	g_free (msg);
+
+	g_free (msg);
+	g_string_free (str, TRUE);
 }
 
 static void
@@ -1497,9 +1574,9 @@ main (int argc, char *argv[])
 	GtkSourceStyleSchemeManager *sm;
 	GtkSourceBuffer *buffer;
 
-	gchar *builtin_lang_dirs[] = {TOP_SRCDIR "/gtksourceview/language-specs", NULL};
-	gchar *builtin_sm_dirs[] = {TOP_SRCDIR "/gtksourceview/language-specs", NULL};
-	gchar **lang_dirs;
+	gchar *builtin_lang_dirs[] = {TOP_SRCDIR "/data/language-specs", NULL};
+	gchar *builtin_sm_dirs[] = {TOP_SRCDIR "/data/styles", NULL};
+	gchar **dirs;
 	const gchar * const * schemes;
 	gboolean use_default_paths = FALSE;
 
@@ -1524,14 +1601,14 @@ main (int argc, char *argv[])
 // 	gdk_window_set_debug_updates (TRUE);
 
 	/* we do not use defaults so we don't need to install the library */
-	lang_dirs = use_default_paths ? NULL : builtin_lang_dirs;
+	dirs = use_default_paths ? NULL : builtin_lang_dirs;
 	lm = gtk_source_language_manager_get_default ();
-	gtk_source_language_manager_set_search_path (lm, lang_dirs);
+	gtk_source_language_manager_set_search_path (lm, dirs);
 
-	lang_dirs = use_default_paths ? NULL : builtin_sm_dirs;
+	dirs = use_default_paths ? NULL : builtin_sm_dirs;
 
 	sm = gtk_source_style_scheme_manager_get_default ();
-	gtk_source_style_scheme_manager_set_search_path (sm, lang_dirs);
+	gtk_source_style_scheme_manager_set_search_path (sm, dirs);
 
 	if (!use_default_paths)
 		gtk_source_style_scheme_manager_append_search_path (sm, TOP_SRCDIR "/tests/test-scheme.xml");
